@@ -1,7 +1,8 @@
 use crate::data::input_state::InputState;
-use crate::data::pointing::{Pointing, Sizing};
+use crate::data::pointing::{KusaPoint, KusaSize};
 use crate::grid::Grid;
 use crate::paint_tool::pen::*;
+use crate::paint_tool::screen_to_image;
 use crate::paint_tool::PaintOperation;
 use crate::paint_tool::PaintTool;
 use crate::piston_wrapper::kusa_image::KusaImage;
@@ -25,7 +26,7 @@ pub fn show_window(mut settings: Settings, k_image: &mut KusaImage) {
 
     // let texture = create_texture(&settings.image_file, &mut window);
     let mut input_state = InputState::default();
-    let mut k_mouse_cursor = Pointing::default();
+    let mut k_mouse_cursor = KusaPoint::default();
     let mut paint_tool = match settings.paint_tool.as_str() {
         "pen" => Pen {},
         _ => Pen {},
@@ -62,8 +63,8 @@ pub fn show_window(mut settings: Settings, k_image: &mut KusaImage) {
             count_to_reload += 1;
         }
         // マウスカーソルの座標を補足するぜ☆（＾～＾）
-        e.mouse_cursor(|pos| {
-            k_mouse_cursor = Pointing::from_pos(pos);
+        e.mouse_cursor(|coord| {
+            k_mouse_cursor = KusaPoint::from_coord(coord);
         });
 
         // 📖 [Event](http://docs.piston.rs/piston_window/piston_window/enum.Event.html)
@@ -71,8 +72,10 @@ pub fn show_window(mut settings: Settings, k_image: &mut KusaImage) {
         // 📖 [PressEvent](https://docs.piston.rs/piston_window/piston_window/trait.PressEvent.html)
         if let Some(_button) = e.press_args() {
             input_state.is_mouse_pressed = true;
-            input_state.pressed_coord = k_mouse_cursor.clone();
+            input_state.pressed_point = k_mouse_cursor.clone();
             //println!("Trace   | ボタンが押されたぜ☆（＾～＾） {:?}", pressed_pos);
+            input_state.previous_point.x = input_state.pressed_point.x;
+            input_state.previous_point.y = input_state.pressed_point.y;
 
             paint_tool.on_mouse_pressed(&settings, &input_state, k_image);
         }
@@ -80,26 +83,41 @@ pub fn show_window(mut settings: Settings, k_image: &mut KusaImage) {
         // TODO ⚡Mouse move
         // 📖 [MouseRelativeEvent](https://docs.piston.rs/piston_window/piston_window/trait.MouseRelativeEvent.html)
         if let Some(coord) = e.mouse_relative_args() {
+            let dx = coord[0];
+            let dy = coord[1];
+
             if input_state.is_mouse_pressed {
-                let dx = coord[0];
-                let dy = coord[1];
-                input_state.pressed_coord.x += dx;
-                input_state.pressed_coord.y += dy;
+                input_state.moved_vector.x += dx;
+                input_state.moved_vector.y += dy;
                 //println!(
                 //    "Trace   | マウス移動中☆（＾～＾） ({:?}, {:?}) ({:?}, {:?})",
                 //    dx, dy, pressed_pos.x, pressed_pos.y
                 //);
             }
 
-            paint_tool.on_mouse_moved(&settings, &input_state, k_image);
+            if paint_tool.on_mouse_moved(&settings, &input_state, k_image) {
+                if input_state.is_mouse_pressed {
+                    // 更新
+                    input_state.previous_point.x += input_state.moved_vector.x;
+                    input_state.previous_point.y += input_state.moved_vector.y;
+                    input_state.moved_vector.x = 0.0;
+                    input_state.moved_vector.y = 0.0;
+                }
+            }
         }
 
         // ⚡Mouse button released
         // 📖 [ReleaseEvent](https://docs.piston.rs/piston_window/piston_window/trait.ReleaseEvent.html)
         if let Some(_button) = e.release_args() {
             //println!("Trace   | ボタンを離したぜ☆（＾～＾）");
-            input_state.is_mouse_pressed = false;
             paint_tool.on_mouse_released(&settings, &input_state, k_image);
+            input_state.is_mouse_pressed = false;
+            input_state.pressed_point.x = 0.0;
+            input_state.pressed_point.y = 0.0;
+            input_state.moved_vector.x = 0.0;
+            input_state.moved_vector.y = 0.0;
+            input_state.previous_point.x = 0.0;
+            input_state.previous_point.y = 0.0;
         }
 
         // ⚡Window paint
@@ -112,7 +130,7 @@ pub fn show_window(mut settings: Settings, k_image: &mut KusaImage) {
             */
 
             // 点を１個描くぜ☆（＾～＾）データとしての保存は別のところでやってるぜ☆（＾～＾）
-            // let sizing = Sizing::diff(&cursor, &pressed_pos);
+            // let sizing = KusaSize::diff(&cursor, &pressed_pos);
 
             /*
             if sizing.is_longer_width() {
@@ -156,18 +174,24 @@ pub fn show_window(mut settings: Settings, k_image: &mut KusaImage) {
             // TODO 今引こうとしている線を、データに描き込まずに画面に表示したいぜ☆（＾～＾）
 
             // グリッド
-            Grid::draw(&settings, &Sizing::load_canvas(&settings), &c, g);
+            let canvas_size = KusaSize {
+                width: settings.image_width as f64 * settings.canvas_dot_width,
+                height: settings.image_height as f64 * settings.canvas_dot_height,
+            };
+            Grid::draw(&settings, &canvas_size, &c, g);
 
             // TODO 座標を表示したいぜ☆（＾～＾）
-            text::Text::new_color([0.0, 0.0, 0.0, 1.0], 32)
-                .draw(
-                    &format!("cell({:.0}, {:.0})", k_mouse_cursor.x, k_mouse_cursor.y),
-                    &mut glyphs,
-                    &c.draw_state,
-                    c.transform.trans(10.0, 30.0), // y位置を揃えるのはむずかしいぜ☆（＾～＾）
-                    g,
-                )
-                .unwrap();
+            if let Some(coord) = screen_to_image(&settings, &k_mouse_cursor) {
+                text::Text::new_color([0.0, 0.0, 0.0, 1.0], 32)
+                    .draw(
+                        &format!("xy({}, {})", coord.x, coord.y),
+                        &mut glyphs,
+                        &c.draw_state,
+                        c.transform.trans(10.0, 30.0), // y位置を揃えるのはむずかしいぜ☆（＾～＾）
+                        g,
+                    )
+                    .unwrap();
+            }
 
             // Update glyphs before rendering.
             glyphs.factory.encoder.flush(device);
